local DEFAULT_CHARACTER_COLOR = Color.new(255, 0, 255, 100)
local COUNTERABLE_COLOR = Color.new(255, 255, 0, 100)
local HITBOX_DISABLED_COLOR = Color.new(0, 0, 0, 0)

local DEFAULT_TILE_COLOR = Color.new(0, 0, 0, 0)
local TILE_RESERVED_COLOR = Color.new(50, 50, 50, 100)

function augment_init(augment)
  local player = augment:owner()
  local component = player:create_component(Lifetime.Scene)
  local tracked_entities = {}

  local field
  local texture = Resources.load_texture("bg.png")

  local function attach_to_entity(e)
    local id = e:id()

    tracked_entities[id] = true

    e:on_delete(function()
      tracked_entities[id] = nil
    end)

    local node = e:sprite():create_node()
    node:set_texture(texture)
    node:set_width(32)
    node:set_height(e:height())
    node:set_offset(-16, -e:height())
    node:set_color(DEFAULT_CHARACTER_COLOR)
    node:set_color_mode(ColorMode.Multiply)
    node:set_layer(999998)

    local component = e:create_component(Lifetime.Scene)
    local counter_counter = 0

    component.on_update_func = function()
      if e:deleted() then
        component:eject()
        return
      end

      node:set_height(e:height())
      node:set_offset(-16, -e:height())
      node:set_color_mode(ColorMode.Multiply)

      if e:counterable() then
        node:set_color(COUNTERABLE_COLOR)
        counter_counter = counter_counter + 1
      else
        node:set_color(DEFAULT_CHARACTER_COLOR)

        if counter_counter > 0 then
          print(e:name() .. " was counterable for " .. counter_counter .. " frames")
          counter_counter = 0
        end
      end

      if not e:hitbox_enabled() then
        node:set_color(HITBOX_DISABLED_COLOR)
      end
    end
  end

  local function add_tile_marker(tile)
    if tile:state() == TileState.Hidden then
      return
    end

    local node = Artifact.new()
    node:set_facing(Direction.Right)
    node:set_texture(texture)
    node:sprite():set_layer(999999)
    node:set_color(DEFAULT_TILE_COLOR)
    node:set_offset(-20, -15)

    local component = node:create_component(Lifetime.Scene)
    local sprite = node:sprite()

    component.on_update_func = function()
      sprite:set_width(40)
      sprite:set_height(30)

      if tile:is_reserved() then
        sprite:set_color(TILE_RESERVED_COLOR)
      else
        sprite:set_color(DEFAULT_TILE_COLOR)
      end
    end

    field:spawn(node, tile)
  end

  component.on_update_func = function()
    if not field then
      field = player:field()

      field:find_tiles(function(tile)
        add_tile_marker(tile)
        return false
      end)
    end

    field:find_entities(function(e)
      if tracked_entities[e:id()] then
        return false
      end

      local character = Character.from(e)
      if character then
        attach_to_entity(character)
        return false
      end

      local obstacle = Obstacle.from(e)
      if obstacle then
        attach_to_entity(obstacle)
        return false
      end

      return false
    end)
  end
end
